@@ -1,6 +1,6 @@
 package B::Debug;
 
-our $VERSION = '1.15';
+our $VERSION = '1.20';
 
 use strict;
 require 5.006;
@@ -181,7 +181,6 @@ sub B::SV::debug {
     printf <<'EOT', class($sv), $$sv, $sv->REFCNT;
 %s (0x%x)
 	REFCNT		%d
-	FLAGS		0x%x
 EOT
     printf "\tFLAGS\t\t0x%x", $sv->FLAGS;
     if ($have_B_Flags) {
@@ -203,9 +202,10 @@ sub B::PV::debug {
     my ($sv) = @_;
     $sv->B::SV::debug();
     my $pv = $sv->PV();
-    printf <<'EOT', cstring($pv), length($pv);
+    printf <<'EOT', cstring($pv), $sv->CUR, $sv->LEN;
 	xpv_pv		%s
 	xpv_cur		%d
+	xpv_len		%d
 EOT
 }
 
@@ -258,17 +258,23 @@ sub B::CV::debug {
     my ($padlist) = $sv->PADLIST;
     my ($file) = $sv->FILE;
     my ($gv) = $sv->GV;
-    printf <<'EOT', $$stash, $$start, $$root, $$gv, $file, $sv->DEPTH, $padlist, ${$sv->OUTSIDE};
+    printf <<'EOT', $$stash, $$start, $$root;
 	STASH		0x%x
 	START		0x%x
 	ROOT		0x%x
-	GV		0x%x
+EOT
+    if ( $]>5.017 && ($sv->FLAGS & 0x40000)) { #lexsub
+      printf("\tNAME\t%%s\n", $sv->NAME);
+    } else {
+      printf("\tGV\t%0x%x\t%s\n", $$gv, $gv->SAFENAME);
+    }
+    printf <<'EOT', $file, $sv->DEPTH, $padlist, ${$sv->OUTSIDE};
 	FILE		%s
 	DEPTH		%d
 	PADLIST		0x%x
 	OUTSIDE		0x%x
 EOT
-    printf("\tOUTSIDE_SEQ\t%d\n", , $sv->OUTSIDE_SEQ) if $] > 5.007;
+    printf("\tOUTSIDE_SEQ\t%d\n", $sv->OUTSIDE_SEQ) if $] > 5.007;
     if ($have_B_Flags) {
       my $SVt_PVCV = $] < 5.010 ? 12 : 13;
       printf("\tCvFLAGS\t0x%x\t%s\n", $sv->CvFLAGS,
@@ -285,11 +291,16 @@ EOT
 sub B::AV::debug {
     my ($av) = @_;
     $av->B::SV::debug;
+    _array_debug($av);
+}
+
+sub _array_debug {
+    my ($av) = @_;
     # tied arrays may leave out FETCHSIZE
     my (@array) = eval { $av->ARRAY; };
     print "\tARRAY\t\t(", join(", ", map("0x" . $$_, @array)), ")\n";
     my $fill = eval { scalar(@array) };
-    if ($Config{'useithreads'}) {
+    if ($Config{'useithreads'} && class($av) ne 'PADLIST') {
       printf <<'EOT', $fill, $av->MAX, $av->OFF;
 	FILL		%d
 	MAX		%d
@@ -350,7 +361,16 @@ EOT
 sub B::SPECIAL::debug {
     my $sv = shift;
     my $i = ref $sv ? $$sv : 0;
-    print exists $specialsv_name[$i] ? $specialsv_name[$i] : "", "\n";
+    print defined $specialsv_name[$i] ? $specialsv_name[$i] : "", "\n";
+}
+
+sub B::PADLIST::debug {
+    my ($padlist) = @_;
+    printf <<'EOT', class($padlist), $$padlist, $padlist->REFCNT;
+%s (0x%x)
+	REFCNT		%d
+EOT
+    _array_debug($padlist);
 }
 
 sub compile {
@@ -373,7 +393,8 @@ B::Debug - Walk Perl syntax tree, printing debug info about ops
 
 =head1 SYNOPSIS
 
-	perl -MO=Debug[,OPTIONS] foo.pl
+        perl -MO=Debug foo.pl
+        perl -MO=Debug,-exec foo.pl
 
 =head1 DESCRIPTION
 
@@ -392,7 +413,7 @@ Reini Urban C<rurban@cpan.org>
 =head1 LICENSE
 
 Copyright (c) 1996, 1997 Malcolm Beattie
-Copyright (c) 2008, 2010 Reini Urban
+Copyright (c) 2008, 2010, 2013 Reini Urban
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of either:
@@ -413,6 +434,7 @@ Copyright (c) 2008, 2010 Reini Urban
     distribution. You should also have received a copy of the GNU General
     Public License, in the file named "Copying". If not, you can get one
     from the Perl distribution or else write to the Free Software Foundation,
-    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 =cut
+
